###############################################################
###    IPG - IDSIA
###    Authors: L. Azzimonti, G. Corani
###    Reference: "Hierarchical Multinomial-Dirichlet model for the estimation of conditional probability tables",
###    Proceedings of 2017 IEEE International Conference on Data Mining (ICDM 2017)
###    Date: August 22, 2017
###
###    This code is for demostration purpose only
###    for licensing contact laura@idsia.ch
###    COPYRIGHT (C) 2017 IDSIA
###############################################################

library(entropy)
library(bnlearn)
library(gtools)

source("src/VAHierTest.R")
source("src/MCHierTest.R")
source("src/convert2array.R")
source("src/compute_loglike.R")
source("src/compute_mse.R")
source("src/predict.R")
source("src/compute_theta_MD_mean.R")
source("src/genData_given_alpha.R")

args=c(1,2,4,"test1")
#args[1] = repetition of the experiment
#args[2] = number of states for child variable
#args[3] = number of states for parent variable
#args[4] = test to be performed. Possible values: "test1", "test2".

set_ripetizioni=as.numeric(args[1])
I=as.numeric(args[2])
J=as.numeric(args[3])


n_vec=10*2^(1:6)
switch=1
s=I
stanFitShrinkage=NULL

if(args[4]=="test1"){
alpha <- rdirichlet(1,rep(1,I))
}else if(arge[4]=="test2"){
alpha <- rdirichlet(1,rep(10^6,I))
}

for(n in n_vec){
time_mat=NULL
theta_mat=NULL
nrep=10
MCris_theta=list()
MSE_theta_mat=NULL
MSE_alpha_mat=NULL
diff_theta_mat=NULL
diff_alpha_mat=NULL

time=NULL
theta=NULL
  
    for(id in 1:nrep){
      data_all=genData_given_alpha(n, I, J,alpha)
      counts=data_all$counts
      data=data_all	
      data$xValues=data_all$xValues
      data$yValues=data_all$yValues
      dataValues=data.frame(xValues = factor(data_all$xValues,levels=1:I), yValues = factor(data_all$yValues,levels=1:J))
      theta_true=as.numeric(data_all$thetaXgivenY)

      theta_freq=as.vector(t(t(counts)/colSums(counts)))
      theta_bayes1=as.vector(t(t(counts+1/(I))/(colSums(counts)+1)))
      theta_bayes10=as.vector(t(t(counts+10/(I))/(colSums(counts)+10)))
      theta_bayesI=as.vector(t(t(counts+s/(I))/(colSums(counts)+s)))
      
      timeMC=system.time({
      MCris=MCHierTest(data$xValues, data$yValues,I, J,stanFitShrinkage)
      })
      stanFitShrinkage=MCris$stanFitShrinkage
      MCris_theta=MCris$theta
      theta_MC=colMeans(MCris$theta)
      MCris_alpha=MCris$alpha
      alpha_MC=colMeans(MCris$alpha)
      time_MC_vec=timeMC[3]
      
      timeVA=system.time({
        VAris=VAHierTest(data$xValues, data$yValues,I, J,stanFitShrinkage)
      })
      VAris_theta=VAris$theta
      theta_VA=colMeans(VAris$theta)
      VAris_alpha=VAris$alpha
      alpha_VA=colMeans(VAris$alpha)
      time_VA_vec=timeVA[3]
      
      err_theta=cbind(theta_freq-theta_true,theta_bayes1-theta_true,theta_bayes10-theta_true,theta_bayesI-theta_true,#theta_int-theta_true, 
                      theta_MC-theta_true,theta_VA-theta_true#, theta_py-theta_true
                      )
      diff_theta=theta_MC-theta_VA
      err_alpha=cbind(t(alpha_MC-alpha), t(alpha_VA-alpha))
      diff_alpha=alpha_MC-alpha_VA
      
      MSE_theta=c(colMeans(err_theta^2), n, I, J)
      MSE_alpha=c(colMeans(err_alpha^2), n, I, J)
      MSE_theta_mat=rbind(MSE_theta_mat, MSE_theta)
      MSE_alpha_mat=rbind(MSE_alpha_mat, MSE_alpha)
      diff_theta_mat=rbind(diff_theta_mat, c(diff_theta, n, I, J))
      diff_alpha_mat=rbind(diff_alpha_mat, c(diff_alpha, n, I, J))
      time_mat=rbind(time_mat,c(time_MC_vec,time_VA_vec,n,I,J))
    }
    


colnames(MSE_theta_mat)=c("ML","BAYES1","BAYES10","BAYESI", "MC","VA", "N", "I", "J")
colnames(MSE_alpha_mat)=c("MC", "VA", "N", "I", "J")
colnames(time_mat)=c( "MC","VA",  "N", "I", "J")

save(file=paste("ris_MSE_",args[4],"_n_",which(n_vec==n),"_I_",I,"_J_",J ,"_rep_",set_ripetizioni,".Rdata",sep=""), MSE_theta_mat, MSE_alpha_mat,time_mat)

}