###############################################################
###    IPG - IDSIA
###    Authors: L. Azzimonti, G. Corani
###    Reference: "Hierarchical Multinomial-Dirichlet model for the estimation of conditional probability tables",
###    Proceedings of 2017 IEEE International Conference on Data Mining (ICDM 2017)
###    Date: August 22, 2017
###
###    This code is for demostration purpose only
###    for licensing contact laura@idsia.ch
###    COPYRIGHT (C) 2017 IDSIA
###############################################################

VAHierTest <- function(xValues, zValues,xStates=2,zStates=2,stanFitShrinkage=NULL) {
  trainingSamples=length(xValues)

  library(MCMCpack)  
  library(rstan)
  rstan_options(auto_write = TRUE)
  options(mc.cores = parallel::detectCores())
  
  zCounts <- as.numeric(table(factor(zValues, levels = 1:zStates)))
  
  xzCounts <- matrix (nrow = zStates, ncol = xStates)
  for (i in 1:zStates){
    xzCounts[i,] <- as.numeric(table(factor(xValues[zValues==i], levels = 1:xStates)))
  }
    dataList = list(
    zStates = zStates,
    xStates = xStates,
    Ntotal = trainingSamples,
    zValues = zValues,
    xValues = xValues,
    alpha0 = rep(1,xStates), 
    s =  xStates
  )
  
    stanModel <-  stan_model(file = 'src/dirHierShrinkage.stan')
    stanVb <-  vb(stanModel, data =  dataList)
    stanResultsShrinkage<- extract(stanVb, permuted = TRUE)
  
  
   postThetaXgivenZ=NULL
   for(k in 1:dim(stanResultsShrinkage$thetaX)[2]){
   postThetaXgivenZ=cbind(postThetaXgivenZ,stanResultsShrinkage$thetaX[,k,])
   }

  return(list(theta=postThetaXgivenZ, thetaZ=stanResultsShrinkage$thetaZ,alpha=stanResultsShrinkage$alpha,stanFitShrinkage=stanFitShrinkage))

}

