
data {
    int<lower=2> zStates;
    int<lower=2> xStates;
    int Ntotal; //number of instances
    int<lower=1,upper=zStates> zValues[Ntotal] ;
    int<lower=1,upper=xStates> xValues[Ntotal];
    vector[xStates] alpha0; //the highest-level Dir coefficients
    real<lower=0> s; //amount of local smoothing. No prior on s for the moment, to be added later however.
   
  }


parameters {
  simplex[zStates] thetaZ;
  simplex[xStates] thetaX[zStates];
  simplex[xStates] alpha;//the prior  for the local states
}

  model {

    //sample the thetaZ from a Dir(1,1,1,..zStates)
    thetaZ ~ dirichlet (rep_vector(1.0,zStates));
    
    //we treat alpha0 as a fixed vector of ones, not sure if it should be treated as 
    //a parameter and thus be given  a prior.
    alpha   ~ dirichlet (alpha0);  

    for (z in 1:zStates) 
      //sample the thetaXY from a Dir(1,1,1,...1)
      thetaX[z] ~ dirichlet (s*alpha);

    
    for ( i in 1:Ntotal ) {
      zValues[i] ~ categorical (thetaZ);
      xValues[i] ~ categorical (thetaX[zValues[i]]);
    }
  
  }
  
  
  
